import socket
import typing
from datetime import datetime, timezone
from enum import Enum


class SyslogFacility(Enum):
    KERN = 0
    USER = 1
    MAIL = 2
    DAEMON = 3
    AUTH = 4
    SYSLOG = 5
    LPR = 6
    NEWS = 7
    UUCP = 8
    CRON = 9
    AUTHPRIV = 10
    FTP = 11
    NTP = 12
    SECURITY = 13
    CONSOLE = 14
    SOLARIS_CRON = 15
    LOCAL0 = 16
    LOCAL1 = 17
    LOCAL2 = 18
    LOCAL3 = 19
    LOCAL4 = 20
    LOCAL5 = 21
    LOCAL6 = 22
    LOCAL7 = 23


class SyslogSeverity(Enum):
    EMERGENCY = 0
    ALERT = 1
    CRITICAL = 2
    ERROR = 3
    WARNING = 4
    NOTICE = 5
    INFO = 6
    DEBUG = 7


def get_c6t_version() -> str:
    return "0.0.0"


def get_local_ip_address() -> str:
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect(("8.8.8.8", 80))
    ip_address = s.getsockname()[0]
    s.close()
    return typing.cast(str, ip_address)


def get_current_time() -> str:
    now = datetime.now(timezone.utc).astimezone()
    formatted_time = now.strftime("%b %d %Y %H:%M:%S.%f")[:-3] + now.strftime("%z")
    return formatted_time


def get_syslog_priority(syslog_facility: int, syslog_severity: int) -> int:
    return syslog_facility * 8 + syslog_severity


def create_cef_message(syslog_facility: int, syslog_severity: int, message: str) -> str:
    current_time = get_current_time()
    ip_address = get_local_ip_address()
    syslog_priority = get_syslog_priority(
        syslog_facility=syslog_facility, syslog_severity=syslog_severity
    )
    c6t_version = get_c6t_version()
    return f"<{syslog_priority}>{current_time} {ip_address} CEF:0|Contrast Security|Contrast - c6t|{c6t_version}|TEST|{message}"


def send_udp_test_message(
    udp_ip: str, udp_port: int, syslog_facility: int, syslog_severity: int, message: str
) -> None:
    """
    Send a UDP message to the specified IP and port.
    """
    address = (udp_ip, udp_port)
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    cef_message = create_cef_message(
        syslog_facility=syslog_facility,
        syslog_severity=syslog_severity,
        message=message,
    )
    sock.sendto(cef_message.encode(), address)
    print(f"Sent message to {address}: {cef_message}")



import pytest
from unittest.mock import MagicMock, patch, mock_open
from pathlib import Path
from lxml import etree

from c6t.api.maven_repo import (
    ChecksumVerifier,
    FileDownloader,
    MavenMetadataHandler,
    JavaAgentDownloader,
)

MAVEN_PACKAGE_NAMESPACE = "com/contrastsecurity/contrast-agent"


@pytest.fixture
def mock_downloader() -> MagicMock:
    return MagicMock(spec=FileDownloader)


@pytest.fixture
def mock_verifier() -> MagicMock:
    return MagicMock(spec=ChecksumVerifier)


@pytest.fixture
def mock_metadata_handler(mock_downloader: MagicMock) -> MavenMetadataHandler:
    return MavenMetadataHandler(mock_downloader, MAVEN_PACKAGE_NAMESPACE)


@pytest.fixture
def agent_downloader(
    mock_downloader: MagicMock,
    mock_verifier: MagicMock,
    mock_metadata_handler: MagicMock,
) -> JavaAgentDownloader:
    return JavaAgentDownloader(mock_downloader, mock_verifier, mock_metadata_handler)


def test_verify_file_checksum_success(mock_verifier: MagicMock) -> None:
    mock_verifier.verify_file_checksum.return_value = True
    result = ChecksumVerifier.verify_file_checksum(
        Path("tests/data/testfile"), Path("tests/data/checksumfile"), "sha1"
    )
    assert result is True


def test_download_file(mock_downloader: MagicMock) -> None:
    url = "https://example.com/file"
    filename = Path("/path/to/file")
    mock_downloader.download_file(url, filename)
    mock_downloader.download_file.assert_called_once_with(url, filename)


def test_parse_maven_metadata(mock_metadata_handler: MagicMock) -> None:
    metadata_content = """<metadata>
                            <versioning>
                                <latest>1.2.3</latest>
                            </versioning>
                          </metadata>"""
    with patch("builtins.open", mock_open(read_data=metadata_content)):
        root = mock_metadata_handler.parse_maven_metadata(Path("metadata.xml"))
        versioning = root.find("versioning")
        assert versioning is not None
        assert versioning.find("latest").text == "1.2.3"


def test_get_latest_version(mock_metadata_handler: MagicMock) -> None:
    root = etree.Element("metadata")
    versioning = etree.SubElement(root, "versioning")
    latest = etree.SubElement(versioning, "latest")
    latest.text = "1.2.3"
    version = mock_metadata_handler.get_latest_version(root)
    assert version == "1.2.3"


def test_is_version_in_metadata(mock_metadata_handler: MagicMock) -> None:
    root = etree.Element("metadata")
    versioning = etree.SubElement(root, "versioning")
    versions = etree.SubElement(versioning, "versions")
    version = etree.SubElement(versions, "version")
    version.text = "1.2.3"
    assert mock_metadata_handler.is_version_in_metadata(root, "1.2.3") is True


def test_download_agent(
    agent_downloader: MagicMock,
    mock_metadata_handler: MagicMock,
    mock_verifier: MagicMock,
    mock_downloader: MagicMock,
) -> None:
    repository_url = "https://example.com/maven-repo"
    version = "latest"
    target_dir = Path("/path/to/target/dir")

    # Mocking temporary directory
    with patch("tempfile.TemporaryDirectory") as mock_temp_dir:
        mock_temp_dir_instance = MagicMock()
        mock_temp_dir.return_value = mock_temp_dir_instance
        mock_temp_dir_instance.__enter__.return_value = Path("/temp/dir")

        # Mock the open function for writing files
        with patch("builtins.open", mock_open()) as _:
            # Mock the download and verification steps
            with patch.object(
                mock_metadata_handler,
                "download_maven_metadata",
                return_value=Path("/temp/dir/maven-metadata.xml"),
            ):
                with patch.object(
                    mock_metadata_handler,
                    "download_maven_metadata_checksum",
                    return_value=Path("/temp/dir/maven-metadata.xml.sha1"),
                ):
                    with patch.object(
                        mock_metadata_handler,
                        "parse_maven_metadata",
                        return_value=etree.Element("root"),
                    ):
                        with patch.object(
                            mock_metadata_handler,
                            "get_latest_version",
                            return_value="1.2.3",
                        ):
                            mock_downloader.download_file.return_value = None
                            mock_verifier.verify_file_checksum.return_value = True

                            # Mock shutil.copy to avoid actual file operations
                            with patch("shutil.copy") as mock_copy:
                                result = agent_downloader.download_agent(
                                    repository_url, version, target_dir
                                )
                                assert result == target_dir / "contrast.jar"
                                mock_downloader.download_file.assert_called()
                                mock_verifier.verify_file_checksum.assert_called()
                                mock_copy.assert_called_once_with(
                                    Path("/temp/dir/contrast.jar"),
                                    target_dir / "contrast.jar",
                                )



from typing import Optional
from pathlib import Path
import sys

import typer
import yaml
from git.repo import Repo
from git.exc import InvalidGitRepositoryError
from jinja2 import Environment, PackageLoader

from rich import print as rprint

import questionary

from c6t.configure.credentials import ContrastAPICredentials
from c6t.ui.auth import ContrastUIAuthManager
from c6t.api.agent_config import AgentConfig
from c6t.api import maven_repo
from c6t.external.integrations.scw.contrast_scw import scw_create, scw_delete
from c6t.tools.udp_server import start_udp_server
from c6t.tools.send_udp_test_message import (
    SyslogFacility,
    SyslogSeverity,
    send_udp_test_message,
)

app = typer.Typer()
integrations_app = typer.Typer()
scw_integration_app = typer.Typer()
tools = typer.Typer()
app.add_typer(
    integrations_app, name="integrations", help="Integrations with other tools"
)
integrations_app.add_typer(scw_integration_app, name="scw", help="Secure Code Warrior")
app.add_typer(tools, name="tools", help="Additional tools")


@app.command("login")
def login(
    profile: str = "default",
    username: Optional[str] = None,
    password: Optional[str] = None,
    environment: Optional[str] = None,
    url: Optional[str] = None,
    organization_uuid: Optional[str] = None,
) -> None:
    """
    Login to the Contrast platform using your UI credentials (username/password).
    This will automatically configure your API credentials and save them to the
    credentials file.

    Args:
        profile: The credentials profile to use (default: "default")
        username: Contrast username (optional)
        password: Contrast password (optional)
        environment: Contrast environment - "trial", "eval", or "enterprise" (optional)
        url: Custom Contrast TeamServer URL for enterprise installations (optional)
        organization_uuid: Specific organization UUID to use (optional)
    """
    contrast_environment = None
    
    if environment:
        environment = environment.lower()
        if environment == "trial":
            contrast_environment = "https://cs004.contrastsecurity.com/Contrast"
        elif environment == "eval":
            contrast_environment = "https://eval.contrastsecurity.com/Contrast"
        elif environment == "enterprise":
            contrast_environment = url if url else questionary.text(
                "Enter your Contrast TeamServer URL:"
            ).ask()
    
    if not contrast_environment:
        contrast_environment = questionary.select(
            "Select your Contrast TeamServer Environment:",
            choices=[
                {
                    "name": "Free Trial",
                    "value": "https://cs004.contrastsecurity.com/Contrast",
                },
                {
                    "name": "Evaluation",
                    "value": "https://eval.contrastsecurity.com/Contrast",
                },
                {"name": "Enterprise", "value": "Enterprise"},
            ],
            default={
                "name": "Free Trial",
                "value": "https://cs004.contrastsecurity.com/Contrast",
            },
        ).ask()

        if contrast_environment == "Enterprise":
            contrast_environment = questionary.text(
                "Enter your Contrast TeamServer URL:"
            ).ask()

    if contrast_environment:
        auth = ContrastUIAuthManager(base_url=contrast_environment)
        # Pre-set the credentials if provided
        if username:
            auth.ui_credentials.username = username
        if password:
            auth.ui_credentials.password = password
        if organization_uuid:
            auth.organization_uuid = organization_uuid
        auth.login(profile=profile)

@app.command("configure")
def configure(profile: str = "default") -> None:
    """
    Configure the c6t CLI credentials.
    This will save your credentials to the credentials file.
    """
    credentials = ContrastAPICredentials()
    credentials.profile = profile
    credentials.base_url = typer.prompt("Contrast TeamServer URL")
    credentials.username = typer.prompt("Contrast username")
    credentials.service_key = typer.prompt("Contrast service key")
    credentials.api_key = typer.prompt("Contrast API key")
    credentials.organization_uuid = typer.prompt("Contrast organization UUID")
    credentials.superadmin = typer.confirm("Contrast superadmin status")
    credentials.write_credentials_to_file()


@app.command("agent-config")
def agent_config(
    profile: str = "default",
    type: str = "yaml",
    path: Optional[str] = None,
    application_name: Optional[str] = None,
    environment: str = "dev",
    language: str = "JAVA",
    include_credentials: bool = True,
) -> None:
    """
    Gets the Contrast Agent YAML config file from TeamServer
    and saves it to the current working directory.
    """

    branch_name: Optional[str] = None
    commit_hash: Optional[str] = None
    committer: Optional[str] = None
    repository: Optional[str] = None

    # Attach to repo in current working directory
    try:
        repo = Repo(".")
        branch_name = str(repo.active_branch)
        commit_hash = str(repo.head.commit)
        committer = str(repo.head.commit.author)
        repository = str(repo.remotes.origin.url)
    except InvalidGitRepositoryError:
        rprint("This command must be run in a git repository.")

    # Get credentials from file
    rprint("Getting agent config from TeamServer...")
    agent_config = AgentConfig(profile=profile)
    raw_yaml_text = agent_config.get_agent_credentials(language=language)
    credentials = yaml.safe_load(raw_yaml_text).get("api")

    # Load Jinga2 template and render using YAML text
    rprint("Rendering agent config...")
    template_env = Environment(
        loader=PackageLoader("c6t", "templates"),
        autoescape=True,
    )

    yaml_template = template_env.get_template("contrast_security.yaml.j2")
    env_template = template_env.get_template("contrast_security_env.yaml.j2")

    rendered_yaml_text = yaml_template.render(
        url=credentials.get("url"),
        api_key=credentials.get("api_key"),
        service_key=credentials.get("service_key"),
        user_name=credentials.get("user_name"),
        application_name=application_name,
        branch_name=branch_name,
        commit_hash=commit_hash,
        committer=committer,
        repository=repository,
        environment=environment,
        include_credentials=include_credentials,
    )

    rendered_env_text = env_template.render(
        url=credentials.get("url"),
        api_key=credentials.get("api_key"),
        service_key=credentials.get("service_key"),
        user_name=credentials.get("user_name"),
        application_name=application_name,
        branch_name=branch_name,
        commit_hash=commit_hash,
        committer=committer,
        repository=repository,
        environment=environment,
        include_credentials=include_credentials,
    )

    rprint("Writing agent config to file...")
    if type == "yaml":
        if path is None:
            path = "contrast_security.yaml"
        agent_config.write_agent_config_to_file(path=path, text=rendered_yaml_text)
        rprint(f"Validating agent config file at {path}...")
        valid = agent_config.validate_agent_config_file_is_valid_yaml(path=path)
        if valid:
            rprint(f"[green]Agent config file at {path} is valid")
        else:
            rprint(f"[red]Agent config file at {path} is invalid")
            sys.exit(1)

    elif type == "env":
        if path is None:
            path = "contrast.env"
        agent_config.write_agent_config_to_file(path=path, text=rendered_env_text)


@app.command("download-agent")
def download_agent(
    language: str = "JAVA",
    repository_url: str = "https://repo1.maven.org/maven2",
    version: str = "latest",
    target_dir: Path = Path("."),
) -> None:
    """
    Download the Contrast agent for the specified language.
    """

    MAVEN_PACKAGE_NAMESPACE = "com/contrastsecurity/contrast-agent"

    if language == "JAVA":
        rprint(f"Downloading Contrast agent ({version})...")
        downloader = maven_repo.FileDownloader()
        verifier = maven_repo.ChecksumVerifier()
        metadata_handler = maven_repo.MavenMetadataHandler(
            downloader=downloader, namespace=MAVEN_PACKAGE_NAMESPACE
        )
        agent_downloader = maven_repo.JavaAgentDownloader(
            downloader=downloader, verifier=verifier, metadata_handler=metadata_handler
        )

        agent_downloader.download_agent(repository_url, version, target_dir)

    else:
        rprint("Unsupported language")


@scw_integration_app.command("create")
def scw(profile: str = "default") -> None:
    """
    Populate vulnerability references with Secure Code Warrior links.
    """
    rprint("Creating SCW links...")
    scw_create(profile=profile)


@scw_integration_app.command("delete")
def scw_reset(profile: str = "default") -> None:
    """
    Delete vulnerability references and reset to the original Contrast links.
    """
    rprint("Deleting SCW links...")
    scw_delete(profile=profile)


@tools.command("udp-server")
def udp_server(
    udp_ip: str = "127.0.0.1",
    udp_port: int = 10514,
    forward_ip: Optional[str] = None,
    forward_port: Optional[int] = None,
) -> None:
    """
    Start a UDP server to receive Contrast agent syslog messages.
    """

    start_udp_server(udp_ip, udp_port, forward_ip, forward_port)


@tools.command("send-udp-message")
def send_udp_message(
    udp_ip: str = "127.0.0.1",
    udp_port: int = 10514,
    syslog_facility: int = SyslogFacility.LOCAL3.value,
    syslog_severity: int = SyslogSeverity.INFO.value,
    message: str = "Test message from c6t.",
) -> None:
    """
    Send a UDP message to the specified IP and port.
    """
    send_udp_test_message(
        udp_ip=udp_ip,
        udp_port=udp_port,
        syslog_facility=syslog_facility,
        syslog_severity=syslog_severity,
        message=message,
    )


if __name__ == "__main__":
    app()



import socket
import threading
from typing import Optional

from rich import print as rprint


def start_udp_server(
    udp_ip: str,
    udp_port: int,
    forward_ip: Optional[str],
    forward_port: Optional[int],
    stop_event: Optional[threading.Event] = None,
) -> None:
    address = (udp_ip, udp_port)
    forward_address = (forward_ip, forward_port)
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock_forward = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind(address)

    rprint(f"UDP server started on {address}")
    if forward_ip and forward_port:
        rprint(f"Forwarding to {forward_address}")
    while not stop_event or not stop_event.is_set():
        data, addr = sock.recvfrom(1024)
        rprint(f"Received message from {addr}: {data.decode()}")
        if forward_ip and forward_port:
            sock_forward.sendto(data, forward_address)
            rprint(f"Forwarded message to {forward_address}")

    sock.close()
    if forward_ip and forward_port:
        sock_forward.close()



import hashlib
from io import BytesIO
from pathlib import Path
import typing
from typing import Any
import shutil
import tempfile

import httpx
from rich.progress import Progress
from rich import print as rprint
from lxml import etree


class ChecksumVerifier:
    @staticmethod
    def verify_file_checksum(
        target_filename: Path, checksum_filename: Path, algorithm: str
    ) -> bool:
        """
        Verify the checksum of a file against a checksum file.
        """
        rprint(f"[cyan]Verifying checksum for {target_filename.name}...")
        with open(target_filename, "rb") as f:
            target_checksum = hashlib.new(algorithm, f.read()).hexdigest()
        with open(checksum_filename, "r") as f:
            checksum = f.read().strip()
        if target_checksum != checksum:
            raise ValueError(f"Checksum does not match for {target_filename.name}.")
        else:
            rprint(f"[green]Checksum verified for {target_filename.name}.")
        return True


class FileDownloader:
    def __init__(self) -> None:
        self.client = httpx.Client()

    def download_file(self, url: str, filename: Path) -> None:
        rprint(f"[cyan]Starting download from: {url}")
        with Progress() as progress:
            task = progress.add_task("[cyan]Downloading...", total=100)
            with self.client.stream("GET", url) as response:
                total_length = int(response.headers["Content-Length"])
                chunk_size = 4096

                with open(filename, "wb") as f:
                    for data in response.iter_bytes(chunk_size=chunk_size):
                        f.write(data)
                        progress.update(task, advance=len(data) / total_length * 100)
        rprint(f"[green]Download completed: {filename.name}")


class MavenMetadataHandler:
    def __init__(self, downloader: FileDownloader, namespace: str):
        self.downloader = downloader
        self.namespace = namespace

    def download_maven_metadata(self, repository_url: str, temp_dir: Path) -> Path:
        url = f"{repository_url}/{self.namespace}/maven-metadata.xml"
        filename = temp_dir / "maven-metadata.xml"
        self.downloader.download_file(url, filename)
        return filename

    def download_maven_metadata_checksum(
        self, repository_url: str, algorithm: str, temp_dir: Path
    ) -> Path:
        filename = temp_dir / f"maven-metadata.xml.{algorithm}"
        url = f"{repository_url}/{self.namespace}/{filename.name}"
        self.downloader.download_file(url, filename)
        return filename

    def parse_maven_metadata(self, metadata_file: Path) -> Any:
        with open(metadata_file, "r") as f:
            metadata = f.read()
        tree = etree.parse(BytesIO(metadata.encode("utf-8")))
        return tree.getroot()

    def get_latest_version(self, root: Any) -> str:
        versioning = root.find("versioning")
        if versioning is not None:
            latest = versioning.find("latest")
            if latest is not None:
                if latest.text is not None:
                    return typing.cast(str, latest.text.strip())
        raise ValueError("Could not find latest version in Maven metadata.")

    def is_version_in_metadata(self, root: Any, version: str) -> bool:
        versioning = root.find("versioning")
        if versioning is not None:
            versions = versioning.find("versions")
            if versions is not None:
                for v in versions:
                    if v.text is not None and v.text.strip() == version:
                        return True
        raise ValueError(f"Version {version} is not in Maven metadata.")


class JavaAgentDownloader:
    def __init__(
        self,
        downloader: FileDownloader,
        verifier: ChecksumVerifier,
        metadata_handler: MavenMetadataHandler,
    ):
        self.downloader = downloader
        self.verifier = verifier
        self.metadata_handler = metadata_handler

    def download_agent(
        self, repository_url: str, version: str, target_dir: Path
    ) -> Path:
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_dir_path = Path(temp_dir)
            maven_metadata_file = self.metadata_handler.download_maven_metadata(
                repository_url, temp_dir_path
            )
            maven_metadata_checksum_file = (
                self.metadata_handler.download_maven_metadata_checksum(
                    repository_url, "sha1", temp_dir_path
                )
            )
            self.verifier.verify_file_checksum(
                maven_metadata_file, maven_metadata_checksum_file, "sha1"
            )

            root = self.metadata_handler.parse_maven_metadata(maven_metadata_file)

            if version == "latest":
                version = self.metadata_handler.get_latest_version(root)
            else:
                if not self.metadata_handler.is_version_in_metadata(root, version):
                    raise ValueError(f"Version {version} is not in Maven metadata.")

            jar_filename = temp_dir_path / "contrast.jar"
            url = f"{repository_url}/{self.metadata_handler.namespace}/{version}/contrast-agent-{version}.jar"
            self.downloader.download_file(url, jar_filename)
            java_agent_checksum_filename = self.download_checksum(
                repository_url, version, temp_dir_path
            )
            self.verifier.verify_file_checksum(
                jar_filename, java_agent_checksum_filename, "sha1"
            )

            target_jar_path = target_dir / "contrast.jar"
            shutil.copy(jar_filename, target_jar_path)
            rprint(f"[green]File copied to {target_jar_path}")

        return target_jar_path

    def download_checksum(
        self, repository_url: str, version: str, temp_dir: Path, algorithm: str = "sha1"
    ) -> Path:
        if version == "latest":
            version = self.metadata_handler.get_latest_version(
                self.metadata_handler.parse_maven_metadata(
                    temp_dir / "maven-metadata.xml"
                )
            )

        java_agent_checksum_filename = (
            temp_dir / f"contrast-agent-{version}.jar.{algorithm}"
        )
        url = f"{repository_url}/{self.metadata_handler.namespace}/{version}/{java_agent_checksum_filename.name}"
        self.downloader.download_file(url, java_agent_checksum_filename)
        return java_agent_checksum_filename



import pytest
from typer.testing import CliRunner
from c6t.cli import app

runner = CliRunner()


def test_main_entrypoint() -> None:
    result = runner.invoke(app, ["--help"])
    assert result.exit_code == 0
    assert "Usage:" in result.output


def test_download_agent() -> None:
    result = runner.invoke(app, ["download-agent", "--language", "JAVA"])
    assert result.exit_code == 0
    assert "Downloading Contrast agent" in result.output


if __name__ == "__main__":
    pytest.main()



import sys

import httpx

import typing
from typing import TypedDict, Any, List, Dict, Optional, cast

import typer

import questionary

from c6t.configure.credentials import ContrastAPICredentials, ContrastUICredentials

class AuthenticationData(TypedDict):
    ui: bool
    username: str
    password: str
    sso: bool


class AuthenticationResponse(TypedDict):
    success: bool
    messages: List[str]
    toggle_enabled: Optional[bool]

class ContrastUIAuthManager:
    def __init__(self, base_url: str):
        self.ui_credentials = ContrastUICredentials()
        self.api_credentials = ContrastAPICredentials()

        self.base_url = base_url
        self.superadmin = False
        self.sso_enabled = False
        self.license_active = False
        self.two_step_verification_enabled = False
        self.client = httpx.Client()
        self.organization_uuid: Optional[str] = None

    def login(self, profile: str) -> None:
        """
        This is the main login function. It calls all the other functions
        """
        self.get_session_cookie()
        self.ui_credentials.get_username_from_user_input()
        self.check_sso(self.ui_credentials.username)
        if self.sso_enabled:
            sys.exit("SSO is enabled. Exiting.")
        self.check_if_license_is_active()
        if not self.license_active:
            sys.exit("TeamServer license is not active. Exiting.")
        self.ui_credentials.get_password_from_user_input()
        self.authenticate(self.ui_credentials.username, self.ui_credentials.password)
        self.set_xsrf_token_header()
        if self.two_step_verification_enabled:
            self.ui_credentials.get_code_from_user_input()
            self.authorize(self.ui_credentials.code)

        self.get_profile_roles()
        if self.ui_credentials.superadmin:
            while True:
                toggle_superadmin = typer.prompt("Toggle Superadmin? (y/n)")
                if toggle_superadmin == "y":
                    self.toggle_superadmin()
                    self.get_superadmin_users_roles()
                    self.organization_uuid = self.get_superadmin_organizations()
                    # TODO: SuperAdmin workflow
                    print("SuperAdmin workflow not implemented yet.")
                    break
                elif toggle_superadmin == "n":
                    print("Not using Superadmin.")
                    break
                else:
                    print("Invalid input.")

        organizations = self.get_organizations()
        self.organization_uuid = self.select_organization(organizations)

        self.api_credentials.profile = profile
        self.api_credentials.base_url = self.base_url
        self.api_credentials.username = self.ui_credentials.username
        self.api_credentials.api_key = self.get_api_key(self.organization_uuid)
        self.api_credentials.service_key = self.get_service_key()
        self.api_credentials.organization_uuid = self.organization_uuid
        self.api_credentials.superadmin = self.superadmin

        self.api_credentials.write_credentials_to_file()

    def get_session_cookie(self) -> None:
        """
        This is a hack to get the SESSION cookie set. It isn't
        completely clear why this is necessary. The cookie is set by the
        server in the response to the request. allow_redirects is set to
        False to prevent receiving the forwarded request. If this is not
        done, a 401 error is received and the SESSION cookie is not set.
        """
        try:
            url = f"{self.base_url}/api/ng/agent/feature"
            response = self.client.get(url, follow_redirects=False)
            if response.status_code != 302:
                # If the response is not a 302, the SESSION cookie is not set.
                response.raise_for_status()
        except httpx.HTTPStatusError as e:
            print(e)
            sys.exit(1)

    def get_information(self) -> None:
        """
        This gets the public information about the server.
        """
        try:
            url = f"{self.base_url}/api/public/ng/information"
            response = self.client.get(url)
            if response.status_code == 200:
                data = response.json()
                if data.get("success"):
                    print(data.get("messages")[0])
                else:
                    print(data.get("messages")[0])
                    sys.exit(1)
            else:
                response.raise_for_status()
        except httpx.HTTPStatusError as e:
            print(e)
            sys.exit(1)

    def check_sso(self, email: str) -> bool:
        """
        This checks if the email address is associated with an SSO account.
        If the response is 200, the email address is associated with an SSO account.
        If the response is 404, the email address is not associated with an SSO account.
        If the response is anything else, raises an exception.
        """
        try:
            url = f"{self.base_url}/api/public/ng/saml/email/{email}"
            response = self.client.get(url)
            if response.status_code == httpx.codes.OK:
                self.sso_enabled = True
            elif response.status_code == httpx.codes.NOT_FOUND:
                self.sso_enabled = False
                data = response.json()
                if data.get("success"):
                    print(data.get("messages")[0])
            else:
                response.raise_for_status()
        except httpx.HTTPStatusError as e:
            print(e)
            sys.exit(1)
        return False

    def check_if_license_is_active(self) -> None:
        """
        This checks if the TeamServer license is active.
        If the response is 200, the license is active.
        If the response is anything else, raises an exception.
        """
        try:
            url = f"{self.base_url}/api/ng/contrast/license/active"
            response = self.client.head(url)
            if response.status_code == 200:
                self.license_active = True
            else:
                response.raise_for_status()
        except httpx.HTTPStatusError as e:
            print(e)
            sys.exit(1)

    def authenticate(self, username: str, password: str) -> None:
        try:
            url = f"{self.base_url}/authenticate.html"
            auth_data: AuthenticationData = {
                "ui": False,
                "username": username,
                "password": password,
                "sso": False,
            }
            response = self.client.post(url, data=auth_data)
            if response.status_code == 200:
                data = cast(AuthenticationResponse, response.json())
                messages: List[str] = []
                if data.get("success"):
                    messages = data["messages"]
                    print(messages[0])
                    self.two_step_verification_enabled = bool(data.get("toggle_enabled", False))
                else:
                    messages = data["messages"]
                    print(messages[0])
                    sys.exit(1)
            else:
                response.raise_for_status()
        except httpx.HTTPStatusError as e:
            print(e)
            sys.exit(1)

    def set_xsrf_token_header(self) -> None:
        """
        This sets the X-XSRF-TOKEN header to the value of the XSRF-TOKEN cookie.
        """
        for cookie in self.client.cookies.jar:
            if cookie.name == "XSRF-TOKEN" and cookie.value:
                self.client.headers.update(
                    {
                        "X-XSRF-TOKEN": cookie.value,
                    }
                )

    def authorize(self, code: str) -> None:
        """
        This authorizes the user with the Two-Step Verification (TSV) code.
        """
        try:
            url = f"{self.base_url}/api/ng/tsv/authorize"
            data = {
                "code": code,
            }
            response = self.client.post(url, json=data)
            if response.status_code == 200:
                data = response.json()
                if data.get("success"):
                    messages: List[str] = typing.cast(List[str], data.get("messages"))
                    print(messages[0])
            else:
                response.raise_for_status()
        except httpx.HTTPStatusError as e:
            print(e)
            sys.exit(1)

    def get_profile_roles(self) -> None:
        """
        This gets the roles for the user.
        """
        try:
            url = f"{self.base_url}/api/ng/profile/roles"
            response = self.client.get(url)
            if response.status_code == 200:
                data = response.json()
                if data.get("success"):
                    print(data.get("messages")[0])
                    if data.get("adminRole") == "SUPERADMIN":
                        self.ui_credentials.superadmin = True
                    else:
                        self.ui_credentials.superadmin = False
                else:
                    print(data.get("messages")[0])
                    sys.exit(1)
            else:
                response.raise_for_status()
        except httpx.HTTPStatusError as e:
            print(e)
            sys.exit(1)

    def get_api_key(self, organization_id: str) -> str:
        """
        This gets the API key for the user.
        """
        try:
            url = f"{self.base_url}/api/ng/{organization_id}/users/keys/apikey"
            response = self.client.get(url)
            if response.status_code == 200:
                data = response.json()
                return typing.cast(str, data.get("api_key"))
            else:
                response.raise_for_status()
            return ""
        except httpx.HTTPStatusError as e:
            print(e)
            sys.exit(1)

    def get_service_key(self) -> str:
        """
        This gets the Service key for the user.
        """
        try:
            url = f"{self.base_url}/api/ng/profile/servicekey"
            response = self.client.get(url)
            if response.status_code == 200:
                data = response.json()
                return typing.cast(str, data.get("service_key"))
            else:
                response.raise_for_status()
            return ""
        except httpx.HTTPStatusError as e:
            print(e)
            sys.exit(1)

    def get_organizations(self) -> List[str]:
        """
        Get the organizations from TeamServer
        """
        url = f"{self.base_url}/api/ng/profile/organizations"

        try:
            response = self.client.get(url)
            if response.status_code == 200:
                data = response.json()
                if data.get("count") > 0:
                    return typing.cast(List[str], data.get("organizations"))
                else:
                    print("No organizations found")
                    sys.exit(1)
            else:
                response.raise_for_status()
            return [""]
        except httpx.HTTPStatusError as e:
            print(e)
            sys.exit(1)

    def select_organization(self, organizations: Any) -> str:
        """
        Ask the user to select an organization by index number or use pre-set organization UUID
        """
        # If organization_uuid is pre-set, validate it exists in the list
        if self.organization_uuid:
            for org in organizations:
                if org.get("organization_uuid") == self.organization_uuid:
                    return self.organization_uuid
            # If pre-set UUID wasn't found, warn user and fall back to interactive selection
            print(f"Warning: Organization UUID {self.organization_uuid} not found in available organizations")

        organization_choices: List[Dict[str, Optional[str]]] = []

        for organization in organizations:
            organization_choices.append(
                {
                    "name": organization.get("name"),
                    "value": organization.get("organization_uuid"),
                }
            )

        if len(organization_choices) == 1:
            return typing.cast(str, organization_choices[0].get("value"))

        organization_uuid = questionary.select(
            message="Select your organization:",
            choices=organization_choices,
        ).ask()

        return typing.cast(str, organization_uuid)

    def toggle_superadmin(self) -> None:
        """
        Toggle the SuperAdmin role for the user.
        This is a hack to get the Contrast UI key cookies set.
        Cookies are set as a side-effect of this request.
        """
        url = f"{self.base_url}/api/ng/profile/toggle"
        try:
            response = self.client.get(url)
            if response.status_code != 200:
                response.raise_for_status()
        except httpx.HTTPStatusError as e:
            print(e)
            sys.exit(1)

    def get_superadmin_users_roles(self) -> List[str]:
        # TODO: This may not be needed.
        # TODO: Determine the meaning of the returned roles.
        """
        Get the SuperAdmin roles for the user
        """
        url = f"{self.base_url}/api/ng/superadmin/users/roles"
        try:
            response = self.client.get(url)
            if response.status_code == 200:
                data = response.json()
                roles = data.get("roles")
                print(roles)
                return typing.cast(List[str], roles)
            else:
                response.raise_for_status()
            return [""]
        except httpx.HTTPStatusError as e:
            print(e)
            sys.exit(1)

    def get_superadmin_organizations(self) -> str:
        """
        Get the SuperAdmin organizations for the user
        """
        url = (
            f"{self.base_url}/api/ng/superadmin/users/"
            f"{self.ui_credentials.username}/organizations"
        )
        params = {
            "includeDefaultOrgs": True,
        }
        try:
            response = self.client.get(url, params=params)
            if response.status_code == 200:
                data = response.json()
                if data.get("success"):
                    print(data.get("messages")[0])
                    for organization in data.get("organizations"):
                        if organization.get("is_superadmin"):
                            organization_uuid = typing.cast(
                                str, organization.get("organization_uuid")
                            )
                            return organization_uuid
                else:
                    print(data.get("messages")[0])
                    sys.exit(1)
            else:
                response.raise_for_status()
            return ""
        except httpx.HTTPStatusError as e:
            print(e)
            sys.exit(1)

    def get_superadmin_api_key(self) -> str:
        """
        Get the SuperAdmin API key for the user
        """
        url = (
            f"{self.base_url}/api/ng/superadmin/users/"
            f"{self.organization_uuid}/keys/apikey"
        )
        try:
            response = self.client.get(url)
            if response.status_code == 200:
                data = response.json()
                api_key = typing.cast(str, data.get("api_key"))
                return api_key
            else:
                response.raise_for_status()
            return ""
        except httpx.HTTPStatusError as e:
            print(e)
            sys.exit(1)



import json
from pathlib import Path
import typer


class ContrastUICredentials:
    def __init__(self):
        self.base_url: str = ""
        self.username: str = ""
        self.superadmin: bool = False
        self.password: str = ""
        self.code: str = ""

    def get_username_from_user_input(self) -> None:
        # Only prompt if username is not already set
        if not self.username:
            username = typer.prompt("Email")
            # TODO: Validate username/email
            self.username = username

    def get_password_from_user_input(self) -> None:
        # Only prompt if password is not already set
        if not self.password:
            password = typer.prompt("Password", hide_input=True)
            # TODO: Validate password
            self.password = password

    def get_code_from_user_input(self) -> None:
        code = typer.prompt("Code")
        # TODO: Validate code
        self.code = code


class ContrastAPICredentials:
    profile: str
    base_url: str
    username: str
    api_key: str
    service_key: str
    organization_uuid: str
    superadmin: bool

    def get_credentials_from_file(self, profile: str) -> None:
        """
        Get credentials from file in JSON format
        """
        credentials_file_path = Path("~/.c6t/credentials.json").expanduser()
        try:
            with open(credentials_file_path, "r") as credentials_file:
                credentials = json.load(credentials_file)
            self.profile = profile
            self.base_url = credentials.get(profile).get("url")
            self.username = credentials.get(profile).get("user_name")
            self.api_key = credentials.get(profile).get("api_key")
            self.service_key = credentials.get(profile).get("service_key")
            self.organization_uuid = credentials.get(profile).get("organization_id")
            self.superadmin = credentials.get(profile).get("superadmin")
        except FileNotFoundError:
            typer.echo(
                "Credentials file not found. Please run `c6t login` or `c6t configure`."
            )
            raise typer.Abort()  # Exit the program

    def write_credentials_to_file(self) -> None:
        """
        Write credentials to file in JSON format
        """
        credentials_file_path = Path("~/.c6t/credentials.json").expanduser()
        credentials_file_path.parent.mkdir(parents=True, exist_ok=True)

        try:
            with open(credentials_file_path, "r") as credentials_file:
                credentials = json.load(credentials_file)
        except FileNotFoundError:
            credentials = {}

        credentials[self.profile] = {
            "url": self.base_url,
            "user_name": self.username,
            "api_key": self.api_key,
            "service_key": self.service_key,
            "organization_id": self.organization_uuid,
            "superadmin": self.superadmin,
        }

        with open(credentials_file_path, "w") as credentials_file:
            json.dump(credentials, credentials_file, indent=4)
